<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Base\Interfaces\NewsItemPresentation;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;

/**
 * ArticleBodyParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base
 * @since   Infomaker\Everyware\Base\ArticleBodyParser 1.0.0
 */
class ArticleBodyParser
{

    /**
     * Contains a sortable map of all items
     *
     * @var array
     */
    private $itemMap = [];

    /**
     * Contains a list of all valid body items
     *
     * @var array
     */
    private $body = [];

    public function __construct(PropertyObject $object, string $propertyName = 'body_raw')
    {
        $transformer = NewsMLTransformerManager::createTransformer($propertyName);
        $bodyElements = $transformer->transform($object);

        foreach ($bodyElements as $index => $bodyElement) {
            if ($bodyElement instanceof Item) {
                $this->addItem($bodyElement);
            }
        }
    }

    private function addItem(Item $item): void
    {
        $this->itemMap[] = $item->getType();
        $this->body[] = $item;
    }

    public function getBody(): array
    {
        return $this->body;
    }

    public function hasType(string $type): bool
    {
        return \in_array($type, $this->itemMap, true);
    }

    public function firstType()
    {
        return collect($this->itemMap)->first();
    }

    public function firstOfType(string $type)
    {
        $allOfType = $this->allOfType($type);

        return array_shift($allOfType);
    }

    public function allPosOfType(string $type): array
    {
        return array_keys($this->itemMap, $type);
    }

    public function removeType(string $type): void
    {
        foreach ($this->allPosOfType($type) as $key) {
            unset($this->itemMap[$key], $this->body[$key]);
        }
    }

    public function allOfType(string $type): array
    {
        $items = [];

        foreach ($this->allPosOfType($type) as $key) {
            $items[] = $this->body[$key];
        }

        return $items;
    }

    public function allOfTypes(array $types): array
    {
        /* the inner empty array covers cases when no loops were made */
        $items = [[]];

        foreach ($types as $type) {
            $items[] = $this->allOfType($type);
        }

        return array_merge(...$items);
    }

    public function generateContent(NewsItemPresentation $presentation): array
    {
        $convertedBody = [];

        foreach ($this->getBody() as $index => $item) {
            if ( ! $item instanceof Item) {
                continue;
            }
            $convertedBody[] = $presentation->generateItem($item->getType(), $item);
        }

        return $convertedBody;
    }
}
