<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Support\Str;

/**
 * Forms
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base
 * @since   Infomaker\Everyware\Base\Forms 1.0.0
 */
class Forms
{

    /**
     * @var string
     */
    private $fieldPrefix;

    /**
     * @var array
     */
    private $fields;


    public function __construct(string $fieldPrefix, array $fields = [])
    {

        $this->fieldPrefix = Str::slug($fieldPrefix);
        $this->fields = $fields;
    }

    public function getFormData(array $data = []): array
    {
        return $this->setupFields(array_replace_recursive($this->fields, $data));
    }

    private function setupFields(array $instance = []): array
    {
        $fieldsValues = [];

        foreach ($this->fields as $field => $value) {
            $fieldsValues[$field] = \is_array($value) ? $this->getFieldMultiValues($field,
                $instance[$field]) : $this->getFieldValues($field, $instance[$field]);
        }

        return $fieldsValues;
    }

    private function getFieldMultiValues(string $field, array $instance = []): array
    {
        $returnVal = [];
        foreach ($instance as $i => $value) {

            if ( ! \is_array($value)) {
                return $this->getFieldValues("{$field}[]", $instance);
            }

            foreach ((array)$value as $settingsName => $settingsValue) {
                $returnVal[$i][$settingsName] = $this->getFieldValues("{$field}[{$i}][{$settingsName}]",
                    $instance[$i][$settingsName]);
            }
        }

        return $returnVal;
    }

    private function getFieldValues(string $field, $value): array
    {

        return [
            'id' => $this->getFieldId($field),
            'name' => $this->getFieldName($field),
            'value' => $value
        ];
    }

    private function getFieldId(string $field_name): string
    {
        return $this->fieldPrefix . '-' . trim(str_replace(['[]', '[', ']'], [
                '',
                '-',
                ''
            ], $field_name), '-');
    }

    private function getFieldName(string $field_name): string
    {
        if (false === $pos = strpos($field_name, '[')) {
            return $this->fieldPrefix . "[{$field_name}]";
        }

        return $this->fieldPrefix . '[' . substr_replace($field_name, '][', $pos, \strlen('['));
    }
}
